@@ -3,7 +3,7 @@
  *
  * Copyright (C) 2005-2012  NTT DATA CORPORATION
  *
- * Version: 0.1.9   2013/02/14
+ * Version: 0.1.10   2013/05/12
  */
 
 #include "internal.h"
@@ -21,37 +21,37 @@
 
 /***** SECTION3: Prototype definition section *****/
 
-static char *ccs_get_absolute_path(struct path *path, char * const buffer,
-				   const int buflen);
-static char *ccs_get_dentry_path(struct dentry *dentry, char * const buffer,
-				 const int buflen);
-static char *ccs_get_local_path(struct dentry *dentry, char * const buffer,
+static char *cs_get_absolute_path(struct path *path, char * const buffer,
+				  const int buflen);
+static char *cs_get_dentry_path(struct dentry *dentry, char * const buffer,
 				const int buflen);
-static char *ccs_get_socket_name(struct path *path, char * const buffer,
-				 const int buflen);
-static int ccs_const_part_length(const char *filename);
+static char *cs_get_local_path(struct dentry *dentry, char * const buffer,
+			       const int buflen);
+static char *cs_get_socket_name(struct path *path, char * const buffer,
+				const int buflen);
+static int cs_const_part_length(const char *filename);
 
 /***** SECTION4: Standalone functions section *****/
 
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 37)
 
 /**
- * ccs_realpath_lock - Take locks for __d_path().
+ * cs_realpath_lock - Take locks for __d_path().
  *
  * Returns nothing.
  */
-static inline void ccs_realpath_lock(void)
+static inline void cs_realpath_lock(void)
 {
 	/* dcache_lock is locked by __d_path(). */
 	/* vfsmount_lock is locked by __d_path(). */
 }
 
 /**
- * ccs_realpath_unlock - Release locks for __d_path().
+ * cs_realpath_unlock - Release locks for __d_path().
  *
  * Returns nothing.
  */
-static inline void ccs_realpath_unlock(void)
+static inline void cs_realpath_unlock(void)
 {
 	/* vfsmount_lock is unlocked by __d_path(). */
 	/* dcache_lock is unlocked by __d_path(). */
@@ -60,22 +60,22 @@ static inline void ccs_realpath_unlock(void)
 #elif LINUX_VERSION_CODE == KERNEL_VERSION(2, 6, 36)
 
 /**
- * ccs_realpath_lock - Take locks for __d_path().
+ * cs_realpath_lock - Take locks for __d_path().
  *
  * Returns nothing.
  */
-static inline void ccs_realpath_lock(void)
+static inline void cs_realpath_lock(void)
 {
 	spin_lock(&dcache_lock);
 	/* vfsmount_lock is locked by __d_path(). */
 }
 
 /**
- * ccs_realpath_unlock - Release locks for __d_path().
+ * cs_realpath_unlock - Release locks for __d_path().
  *
  * Returns nothing.
  */
-static inline void ccs_realpath_unlock(void)
+static inline void cs_realpath_unlock(void)
 {
 	/* vfsmount_lock is unlocked by __d_path(). */
 	spin_unlock(&dcache_lock);
@@ -84,7 +84,7 @@ static inline void ccs_realpath_unlock(void)
 #elif defined(D_PATH_DISCONNECT) && !defined(CONFIG_SUSE_KERNEL)
 
 /**
- * ccs_realpath_lock - Take locks for __d_path().
+ * cs_realpath_lock - Take locks for __d_path().
  *
  * Returns nothing.
  *
@@ -102,18 +102,18 @@ static inline void ccs_realpath_unlock(void)
  * order, and add " && 0" to this "#elif " block if fs/dcache.c uses original
  * order.
  */
-static inline void ccs_realpath_lock(void)
+static inline void cs_realpath_lock(void)
 {
 	spin_lock(caitsith_exports.vfsmount_lock);
 	spin_lock(&dcache_lock);
 }
 
 /**
- * ccs_realpath_unlock - Release locks for __d_path().
+ * cs_realpath_unlock - Release locks for __d_path().
  *
  * Returns nothing.
  */
-static inline void ccs_realpath_unlock(void)
+static inline void cs_realpath_unlock(void)
 {
 	spin_unlock(&dcache_lock);
 	spin_unlock(caitsith_exports.vfsmount_lock);
@@ -122,22 +122,22 @@ static inline void ccs_realpath_unlock(void)
 #else
 
 /**
- * ccs_realpath_lock - Take locks for __d_path().
+ * cs_realpath_lock - Take locks for __d_path().
  *
  * Returns nothing.
  */
-static inline void ccs_realpath_lock(void)
+static inline void cs_realpath_lock(void)
 {
 	spin_lock(&dcache_lock);
 	spin_lock(caitsith_exports.vfsmount_lock);
 }
 
 /**
- * ccs_realpath_unlock - Release locks for __d_path().
+ * cs_realpath_unlock - Release locks for __d_path().
  *
  * Returns nothing.
  */
-static inline void ccs_realpath_unlock(void)
+static inline void cs_realpath_unlock(void)
 {
 	spin_unlock(caitsith_exports.vfsmount_lock);
 	spin_unlock(&dcache_lock);
@@ -150,7 +150,7 @@ static inline void ccs_realpath_unlock(void)
 /***** SECTION6: Dependent functions section *****/
 
 /**
- * ccs_get_absolute_path - Get the path of a dentry but ignores chroot'ed root.
+ * cs_get_absolute_path - Get the path of a dentry but ignores chroot'ed root.
  *
  * @path:   Pointer to "struct path".
  * @buffer: Pointer to buffer to return value in.
@@ -161,8 +161,8 @@ static inline void ccs_realpath_unlock(void)
  * Caller holds the dcache_lock and vfsmount_lock.
  * Based on __d_path() in fs/dcache.c
  */
-static char *ccs_get_absolute_path(struct path *path, char * const buffer,
-				   const int buflen)
+static char *cs_get_absolute_path(struct path *path, char * const buffer,
+				  const int buflen)
 {
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(3, 2, 0)
 	if (buflen < 256)
@@ -176,21 +176,21 @@ static char *ccs_get_absolute_path(struct path *path, char * const buffer,
 	 *
 	 * Unfortunately, __d_path() after applying that commit always returns
 	 * NULL when root is empty. d_absolute_path() is provided for TOMOYO
-	 * 2.x and AppArmor but TOMOYO 1.x does not use it, for TOMOYO 1.x
+	 * 2.x and AppArmor but CaitSith does not use it, for CaitSith
 	 * might be built as a loadable kernel module and there is no warrantee
-	 * that TOMOYO 1.x is recompiled after applying that commit. Also,
+	 * that CaitSith is recompiled after applying that commit. Also,
 	 * I don't want to search /proc/kallsyms for d_absolute_path() because
-	 * I want to keep TOMOYO 1.x architecture independent. Thus, supply
+	 * I want to keep CaitSith architecture independent. Thus, supply
 	 * non empty root like AppArmor's d_namespace_path() did.
 	 */
-	static bool ccs_no_empty;
+	static bool cs_no_empty;
 	char *pos;
 	if (buflen < 256)
 		return ERR_PTR(-ENOMEM);
-	if (!ccs_no_empty) {
+	if (!cs_no_empty) {
 		struct path root = { };
 		pos = caitsith_exports.__d_path(path, &root, buffer,
-						  buflen - 1);
+						buflen - 1);
 	} else {
 		pos = NULL;
 	}
@@ -205,12 +205,12 @@ static char *ccs_get_absolute_path(struct path *path, char * const buffer,
 		spin_unlock(&task->fs->lock);
 		tmp = root;
 		pos = caitsith_exports.__d_path(path, &tmp, buffer,
-						  buflen - 1);
+						buflen - 1);
 		path_put(&root);
 		if (pos)
 			return pos;
 		/* Remember if __d_path() needs non empty root. */
-		ccs_no_empty = true;
+		cs_no_empty = true;
 		pos = ERR_PTR(-EINVAL);
 	}
 	return pos;
@@ -258,7 +258,7 @@ out:
 }
 
 /**
- * ccs_get_dentry_path - Get the path of a dentry.
+ * cs_get_dentry_path - Get the path of a dentry.
  *
  * @dentry: Pointer to "struct dentry".
  * @buffer: Pointer to buffer to return value in.
@@ -268,8 +268,8 @@ out:
  *
  * Based on dentry_path() in fs/dcache.c
  */
-static char *ccs_get_dentry_path(struct dentry *dentry, char * const buffer,
-				 const int buflen)
+static char *cs_get_dentry_path(struct dentry *dentry, char * const buffer,
+				const int buflen)
 {
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 38)
 	if (buflen < 256)
@@ -303,7 +303,7 @@ static char *ccs_get_dentry_path(struct dentry *dentry, char * const buffer,
 }
 
 /**
- * ccs_get_local_path - Get the path of a dentry.
+ * cs_get_local_path - Get the path of a dentry.
  *
  * @dentry: Pointer to "struct dentry".
  * @buffer: Pointer to buffer to return value in.
@@ -311,11 +311,11 @@ static char *ccs_get_dentry_path(struct dentry *dentry, char * const buffer,
  *
  * Returns the buffer on success, an error code otherwise.
  */
-static char *ccs_get_local_path(struct dentry *dentry, char * const buffer,
-				const int buflen)
+static char *cs_get_local_path(struct dentry *dentry, char * const buffer,
+			       const int buflen)
 {
 	struct super_block *sb = dentry->d_sb;
-	char *pos = ccs_get_dentry_path(dentry, buffer, buflen);
+	char *pos = cs_get_dentry_path(dentry, buffer, buflen);
 	if (IS_ERR(pos))
 		return pos;
 	/* Convert from $PID to self if $PID is current thread. */
@@ -375,7 +375,7 @@ out:
 }
 
 /**
- * ccs_get_socket_name - Get the name of a socket.
+ * cs_get_socket_name - Get the name of a socket.
  *
  * @path:   Pointer to "struct path".
  * @buffer: Pointer to buffer to return value in.
@@ -383,8 +383,8 @@ out:
  *
  * Returns the buffer.
  */
-static char *ccs_get_socket_name(struct path *path, char * const buffer,
-				 const int buflen)
+static char *cs_get_socket_name(struct path *path, char * const buffer,
+				const int buflen)
 {
 	struct inode *inode = path->dentry->d_inode;
 	struct socket *sock = inode ? SOCKET_I(inode) : NULL;
@@ -400,7 +400,7 @@ static char *ccs_get_socket_name(struct path *path, char * const buffer,
 }
 
 /**
- * ccs_realpath - Returns realpath(3) of the given pathname but ignores chroot'ed root.
+ * cs_realpath - Returns realpath(3) of the given pathname but ignores chroot'ed root.
  *
  * @path: Pointer to "struct path".
  *
@@ -409,7 +409,7 @@ static char *ccs_get_socket_name(struct path *path, char * const buffer,
  * This function uses kzalloc(), so caller must kfree() if this function
  * didn't return NULL.
  */
-char *ccs_realpath(struct path *path)
+char *cs_realpath(struct path *path)
 {
 	char *buf = NULL;
 	char *name = NULL;
@@ -431,7 +431,7 @@ char *ccs_realpath(struct path *path)
 		buf[buf_len - 1] = '\0';
 		/* Get better name for socket. */
 		if (sb->s_magic == SOCKFS_MAGIC) {
-			pos = ccs_get_socket_name(path, buf, buf_len - 1);
+			pos = cs_get_socket_name(path, buf, buf_len - 1);
 			goto encode;
 		}
 		/* For "pipe:[\$]". */
@@ -449,27 +449,27 @@ char *ccs_realpath(struct path *path)
 			pos = ERR_PTR(-EINVAL);
 		else {
 			/* Get absolute name for the rest. */
-			ccs_realpath_lock();
-			pos = ccs_get_absolute_path(path, buf, buf_len - 1);
-			ccs_realpath_unlock();
+			cs_realpath_lock();
+			pos = cs_get_absolute_path(path, buf, buf_len - 1);
+			cs_realpath_unlock();
 		}
 		if (pos == ERR_PTR(-EINVAL))
-			pos = ccs_get_local_path(path->dentry, buf,
-						 buf_len - 1);
+			pos = cs_get_local_path(path->dentry, buf,
+						buf_len - 1);
 encode:
 		if (IS_ERR(pos))
 			continue;
-		name = ccs_encode(pos);
+		name = cs_encode(pos);
 		break;
 	}
 	kfree(buf);
 	if (!name)
-		ccs_warn_oom(__func__);
+		cs_warn_oom(__func__);
 	return name;
 }
 
 /**
- * ccs_encode2 - Encode binary string to ascii string.
+ * cs_encode2 - Encode binary string to ascii string.
  *
  * @str:     String in binary format. Maybe NULL.
  * @str_len: Size of @str in byte.
@@ -479,7 +479,7 @@ encode:
  * This function uses kzalloc(), so caller must kfree() if this function
  * didn't return NULL.
  */
-char *ccs_encode2(const char *str, int str_len)
+char *cs_encode2(const char *str, int str_len)
 {
 	int i;
 	int len;
@@ -515,7 +515,7 @@ char *ccs_encode2(const char *str, int str_len)
 }
 
 /**
- * ccs_encode - Encode binary string to ascii string.
+ * cs_encode - Encode binary string to ascii string.
  *
  * @str: String in binary format. Maybe NULL.
  *
@@ -524,19 +524,19 @@ char *ccs_encode2(const char *str, int str_len)
  * This function uses kzalloc(), so caller must kfree() if this function
  * didn't return NULL.
  */
-char *ccs_encode(const char *str)
+char *cs_encode(const char *str)
 {
-	return str ? ccs_encode2(str, strlen(str)) : NULL;
+	return str ? cs_encode2(str, strlen(str)) : NULL;
 }
 
 /**
- * ccs_const_part_length - Evaluate the initial length without a pattern in a token.
+ * cs_const_part_length - Evaluate the initial length without a pattern in a token.
  *
  * @filename: The string to evaluate. Maybe NULL.
  *
  * Returns the initial length without a pattern in @filename.
  */
-static int ccs_const_part_length(const char *filename)
+static int cs_const_part_length(const char *filename)
 {
 	char c;
 	int len = 0;
@@ -557,13 +557,13 @@ static int ccs_const_part_length(const char *filename)
 		case '2':
 		case '3':
 			c = *filename++;
-			if (c < '0' || c > '7')
-				break;
-			c = *filename++;
-			if (c < '0' || c > '7')
-				break;
-			len += 4;
-			continue;
+		if (c < '0' || c > '7')
+			break;
+		c = *filename++;
+		if (c < '0' || c > '7')
+			break;
+		len += 4;
+		continue;
 		}
 		break;
 	}
@@ -571,32 +571,32 @@ static int ccs_const_part_length(const char *filename)
 }
 
 /**
- * ccs_fill_path_info - Fill in "struct ccs_path_info" members.
+ * cs_fill_path_info - Fill in "struct cs_path_info" members.
  *
- * @ptr: Pointer to "struct ccs_path_info" to fill in.
+ * @ptr: Pointer to "struct cs_path_info" to fill in.
  *
  * Returns nothing.
  *
- * The caller sets "struct ccs_path_info"->name.
+ * The caller sets "struct cs_path_info"->name.
  */
-void ccs_fill_path_info(struct ccs_path_info *ptr)
+void cs_fill_path_info(struct cs_path_info *ptr)
 {
 	const char *name = ptr->name;
 	const int len = strlen(name);
 	ptr->total_len = len;
-	ptr->const_len = ccs_const_part_length(name);
+	ptr->const_len = cs_const_part_length(name);
 	ptr->hash = full_name_hash(name, len);
 }
 
 /**
- * ccs_get_exe - Get ccs_realpath() of current process.
+ * cs_get_exe - Get cs_realpath() of current process.
  *
- * Returns the ccs_realpath() of current process on success, NULL otherwise.
+ * Returns the cs_realpath() of current process on success, NULL otherwise.
  *
  * This function uses kzalloc(), so the caller must kfree()
  * if this function didn't return NULL.
  */
-char *ccs_get_exe(void)
+char *cs_get_exe(void)
 {
 	struct mm_struct *mm;
 #if LINUX_VERSION_CODE < KERNEL_VERSION(3, 7, 0)
@@ -612,13 +612,13 @@ char *ccs_get_exe(void)
 	down_read(&mm->mmap_sem);
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(3, 7, 0)
 	if (mm->exe_file) {
-		cp = ccs_realpath(&mm->exe_file->f_path);
+		cp = cs_realpath(&mm->exe_file->f_path);
 		done = true;
 	}
 #else
 	for (vma = mm->mmap; vma; vma = vma->vm_next) {
 		if ((vma->vm_flags & VM_EXECUTABLE) && vma->vm_file) {
-			cp = ccs_realpath(&vma->vm_file->f_path);
+			cp = cs_realpath(&vma->vm_file->f_path);
 			done = true;
 			break;
 		}
@@ -633,20 +633,20 @@ task_has_no_mm:
 }
 
 /**
- * ccs_get_exename - Get ccs_realpath() of current process.
+ * cs_get_exename - Get cs_realpath() of current process.
  *
- * @buf: Pointer to "struct ccs_path_info".
+ * @buf: Pointer to "struct cs_path_info".
  *
  * Returns true on success, false otherwise.
  *
  * This function uses kzalloc(), so the caller must kfree()
  * if this function returned true.
  */
-bool ccs_get_exename(struct ccs_path_info *buf)
+bool cs_get_exename(struct cs_path_info *buf)
 {
-	buf->name = ccs_get_exe();
+	buf->name = cs_get_exe();
 	if (buf->name) {
-		ccs_fill_path_info(buf);
+		cs_fill_path_info(buf);
 		return true;
 	}
 	return false;
